########################################################################################################################
# Vegan product store software

# This project consists of creating software for managing a vegan product shop.
# The software must have the following features:
# 1) Register new products, with name, quantity, selling price and purchase price.
# 2) List all the products present.
# 3) Record the sales you make.
# 4) Show gross and net profits.
# 5) Show a help menu with all available commands.
########################################################################################################################

#-------------------------------------------------------------------------------
# Name:        Vegan product store software
#-------------------------------------------------------------------------------

class VeganProducts:
    """
    Class for managing a vegan product shop.
    """

    def __init__(self):
        """
        Initialize a new vegan shop.
        """
        self.products = {}  # Dictionary for storing products {product name: [quantity, purchase price, sale price]}
        self.sales = []  # List for recording sales [product name, quantity sold]

    def add_product(self, name, quantity, purchase_price, selling_price):
        """
        Adds a new product to the store.

        Parameters:
        name (str): The name of the product.
        quantity (int): The quantity of the product.
        purchase_price (float): The purchase price of the product.
        selling_price (float): The selling price of the product.

        Raises:
        ValueError: If the quantity, purchase price or selling price is negative.
        """
        if quantity < 0 or purchase_price < 0 or selling_price < 0:
            raise ValueError("The quantity, purchase price and selling price must be positive.")

        if name in self.products:
            # If the product already exists, update the quantity
            self.products[name][0] += quantity
            #print(f"The quantity of '{name}' has been updated to {self.products[name][0]}.")
        else:
            # If the product does not exist, add the product.
            self.products[name] = [quantity, purchase_price, selling_price]
            #print(f"Add {str(quantity) + ' X ' + name}")

    def list_products(self):
        """
        Shows all products in the shop.
        """
        print("Products available in the shop:")
        for name, details in self.products.items():
            # Print the product list
            print(f"{name}      {details[0]}        €{details[2]}")

    def register_sale(self, name, quantity):
        """
        Record a product sale.

        Parameters:
        name (str): The name of the product sold.
        quantity (int): The quantity sold.

        Raises:
        ValueError: If the quantity is negative or if the product does not exist in the store.
        """
        if quantity < 0:
            raise ValueError("The quantity of the sale must be positive.")

        if name in self.products:
            if self.products[name][0] >= quantity:
                self.products[name][0] -= quantity
                self.sales.append([name, quantity])

                for i in self.products:
                    add_new_product = input("Do you want to add a new product to sell? \n Press Y or N:   ")
                    if (add_new_product == 'Y' or add_new_product == 'y'):
                        name = input("Enter the product name: ")
                        quantity = int(input("Enter the available quantity: "))
                        self.products[name][0] -= quantity #sottrae la quantità del prodotto aggiunto alla quantità venduta
                        self.sales.append([name, quantity])
                    else:
                        print("No products to add!")

                print('VENDITA REGISTRATA')
                total_price = 0
                for name, quantity in self.sales:
                    if name in self.products:
                        total_price += self.products[name][2] * quantity
                        print(f"{quantity} X {name} : € {self.products[name][2]}")
                print(f"Totale € {total_price}")

                if self.products[name][0] == 0:
                    del self.products[name]
                    self.products[name].clear()
                    print(f"The product '{name}' has been eliminated from the store.")
            else:
                print(f"Insufficient quantity of'{name}' in the store.")
        else:
            print(f"The product '{name}' does not exist in the shop.")

    def show_profits(self):
        """
        Shows the store's gross and net profits.
        """
        gross_profits = 0
        for sale in self.sales:
            selling_price = self.products[sale[0]][2]
            gross_profits += selling_price * sale[1]
        net_profits = gross_profits
        for name, details in self.products.items():
            purchase_price = details[1]
            net_profits -= purchase_price * details[0]
        print(f"Gross profits: {gross_profits}")
        print(f"Net profits:: {net_profits}")

        #self.profitto_lordo = sum(vendita["importo"] for vendita in self.vendite)
        #self.profitto_netto = self.profitto_lordo - sum(prodotto.quantita * prodotto.prezzo_acquisto
                                                       # for prodotto in self.prodotti)

    def show_help_menu(self):
        """
        Shows the help menu with all available commands.
        """
        print("Available commands:")
        print("1. Add product")
        print("2. Product list")
        print("3. Record sale")
        print("4. Help")
        print("5. Profits")
        print("0. Close")

    def exit(self):
        """
        Exits the program.
        """
        print("Exiting the program.")
        exit()


def main():
    """
    Main function for program execution.
    """
    store = VeganProducts()
    store.show_help_menu()

    while True:
        choice = input("Enter the number corresponding to the desired action: ")

        if choice == "1":
            try:
                name = input("Enter the product name: ")
                quantity = int(input("Enter the available quantity: "))
                purchase_price = float(input("Enter the purchase price: "))
                selling_price = float(input("Enter the sales price: "))
                store.add_product(name, quantity, purchase_price, selling_price)
            except ValueError as e:
                print(e)

        elif choice == "2":
            store.list_products()

        elif choice == "3":
            try:
                name = input("Enter the name of the product sold: ")
                quantity = int(input("Enter the quantity sold: "))
                store.register_sale(name, quantity)
            except ValueError as e:
                print(e)

        elif choice == "4":
            store.show_help_menu()

        elif choice == "5":
            store.show_profits()
        elif choice == "0":
            store.exit()

        else:
            print("Invalid choice. Try again.")

if __name__ == "__main__":
    main()
